"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_1 = require("vscode");
const resources_1 = require("../resources");
const utils_1 = require("../utils");
class default_1 {
    provideHover(doc, pos, token) {
        const tag = utils_1.getTag(doc, pos);
        if (!tag)
            return null;
        // when is component
        if (tag.isOnTagName) {
            const component = resources_1.first(tag.word);
            if (component) {
                const directive = resources_1.first(tag.word);
                if (!directive)
                    return null;
                return new vscode_1.Hover(directive._doc);
            }
            return null;
        }
        // when is property name
        if (tag.isOnAttrName) {
            const directive = resources_1.getDirective(tag);
            if (directive == null)
                return null;
            const property = directive.properties.find(w => w.name === tag.word);
            return property ? new vscode_1.Hover(property._doc) : null;
        }
        // when is property value
        if (tag.isOnAttrValue) {
            const attr = tag.attributes[tag.attrName];
            // when is complex type
            if (attr.value && attr.value.startsWith('{')) {
                const directive = resources_1.getDirective(tag);
                if (directive == null)
                    return null;
                const property = directive.properties.find(w => w.name === attr.name);
                if (property == null || !property.complexType || property.complexType.length === 0)
                    return null;
                const complexTypeProperties = directive.types[property.complexType];
                const complexDirective = complexTypeProperties.find(w => w.name === tag.word);
                return complexDirective ? new vscode_1.Hover(complexDirective._doc) : null;
            }
        }
    }
}
exports.default = default_1;
//# sourceMappingURL=hover-provider.js.map