"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const interfaces_1 = require("../interfaces");
const local_1 = require("./local");
const zh_CN_json_1 = __importDefault(require("./zh-CN.json"));
const en_US_json_1 = __importDefault(require("./en-US.json"));
const vscode_1 = require("vscode");
const fs_1 = require("fs");
const path_1 = require("path");
exports.CONFIG = {
    language: 'zh-CN',
    hover: true,
    isAntd: true,
    isAlain: true,
};
exports.RESOURCES = [];
exports.DIRECTIVE_NAMES = [];
function notNull(value, defaultValue = '') {
    if (value == null)
        return defaultValue;
    return value;
}
function getPure(value) {
    return value.replace(/nz([A-Z])/g, (raw, val) => {
        return val.toLowerCase();
    });
}
function getFullDoc(item) {
    if (item.doc.startsWith('http'))
        return item.doc;
    if (item.lib === 'ng-zorro-antd') {
        return (exports.CONFIG.language === 'zh-CN' ? 'https://ng-zorro.gitee.io' : 'https://ng.ant.design') + item.doc;
    }
    return (exports.CONFIG.language === 'zh-CN' ? 'https://ng-alain.com' : 'https://netlify.ng-alain.com') + item.doc;
}
function genComponentMarkdown(item) {
    if (item == null)
        return '';
    const rows = [exports.CONFIG.isAlain ? `**${local_1.i18n('library')}** ${item.lib}` : null, `**${item.title}**`, item.description];
    if (item.whenToUse) {
        rows.push(`**${local_1.i18n('whenToUse')}**`);
        rows.push(item.whenToUse);
    }
    if (item.doc) {
        rows.push(`[${local_1.i18n('document')}](${getFullDoc(item)})${item.github ? ` － [${local_1.i18n('github')}](${item.github})` : ''}`);
    }
    return rows.filter(w => !!w).join('\n\n');
}
function genPropertyMarkdown(property) {
    if (property == null)
        return '';
    const rows = [];
    if (property.typeDefinition && Array.isArray(property.typeDefinition) && property.typeDefinition.length > 0) {
        rows.push(`**${local_1.i18n('optionalValue')}** ${property.typeDefinition.map((i) => '`' + i.label + '`').join(', ')}`);
    }
    else if (property.typeRaw.length > 0) {
        rows.push(`**${local_1.i18n('type')}** ${property.typeRaw}`);
    }
    if (property.default.length > 0) {
        rows.push(`**${local_1.i18n('defaultValue')}** ${property.default}`);
    }
    rows.push(property.description);
    return rows.join('\n\n');
}
function fixProperty(p) {
    p.pureName = getPure(p.name);
    p.default = notNull(p.default, '-');
    if (p.default === '-')
        p.default = '';
    p.description = notNull(p.description);
    p.isInputBoolean = notNull(p.isInputBoolean, true);
    p.type = notNull(p.type, 'string');
    p.typeRaw = notNull(p.typeRaw, '').replace(/｜/g, '|');
    p.typeDefinition = notNull(p.typeDefinition, []);
    if (Array.isArray(p.typeDefinition)) {
        p.typeDefinition = p.typeDefinition.map((item) => (typeof item === 'string' ? { value: item, label: item } : item));
        p.typeDefinitionSnippetStr = p.typeDefinition.map((i) => i.value).join(',');
    }
    else {
        if (typeof p.typeDefinition === 'object') {
            p.inputType = interfaces_1.InputAttrType.Complex;
        }
        p.typeDefinitionSnippetStr = '';
    }
    p._doc = new vscode_1.MarkdownString(genPropertyMarkdown(p));
}
function INIT(notifier) {
    return __awaiter(this, void 0, void 0, function* () {
        const cog = vscode_1.workspace.getConfiguration(interfaces_1.NAME);
        exports.CONFIG.language = cog.language;
        exports.CONFIG.hover = cog.hover;
        if (vscode_1.workspace.workspaceFolders != null && vscode_1.workspace.workspaceFolders.length > 0) {
            exports.CONFIG.isAntd = false;
            exports.CONFIG.isAlain = false;
            const packageJsonPath = path_1.join(vscode_1.workspace.workspaceFolders[0].uri.fsPath, 'package.json');
            if (!fs_1.existsSync(packageJsonPath)) {
                notifier.notify(`Not found package.json`);
                return null;
            }
            const packageJson = JSON.parse(fs_1.readFileSync(packageJsonPath).toString());
            const VERSIONS = [...Object.keys(packageJson.dependencies), ...Object.keys(packageJson.devDependencies)];
            if (~VERSIONS.indexOf(`ng-zorro-antd`)) {
                exports.CONFIG.isAntd = true;
            }
            if (~VERSIONS.indexOf(`@delon/theme`)) {
                exports.CONFIG.isAntd = true;
                exports.CONFIG.isAlain = true;
            }
        }
        const data = (exports.CONFIG.language === 'en-US' ? en_US_json_1.default : zh_CN_json_1.default);
        if (exports.CONFIG.isAntd) {
            exports.RESOURCES.push(...data.filter(i => i.lib === 'ng-zorro-antd'));
        }
        if (exports.CONFIG.isAlain) {
            exports.RESOURCES.push(...data.filter(i => i.lib !== 'ng-zorro-antd'));
        }
        exports.RESOURCES.forEach(i => {
            i.description = notNull(i.description);
            i.whenToUse = notNull(i.whenToUse);
            i.doc = notNull(i.doc);
            i.title = notNull(i.title);
            i.type = notNull(i.type, 'component');
            i.snippet = notNull(i.snippet, '');
            if (i.snippet.length === 0) {
                i.snippet = i.type === 'component' ? `<__$1>$0</__>` : `<div __$1>$0</div>`;
            }
            i.snippet = i.snippet.replace(/__/g, i.selector);
            i.properties = notNull(i.properties, []);
            i.properties.forEach(p => fixProperty(p));
            if (i.types) {
                Object.keys(i.types).forEach(key => {
                    i.types[key].forEach(p => fixProperty(p));
                });
            }
            i._doc = new vscode_1.MarkdownString(genComponentMarkdown(i));
            exports.DIRECTIVE_NAMES.push(i.selector);
        });
    });
}
exports.INIT = INIT;
function isComponent(text) {
    return text.startsWith('nz-') || exports.DIRECTIVE_NAMES.indexOf(text) !== -1;
}
exports.isComponent = isComponent;
function getDirective(tag) {
    if (isComponent(tag.name)) {
        return first(tag.name);
    }
    const key = Object.keys(tag.attributes).find(key => exports.DIRECTIVE_NAMES.indexOf(tag.attributes[key].name) !== -1);
    if (key) {
        return first(tag.attributes[key].name);
    }
    return null;
}
exports.getDirective = getDirective;
function query(type) {
    return [...exports.RESOURCES].filter(i => i.type === type);
}
exports.query = query;
function first(selector) {
    return exports.RESOURCES.find(w => w.selector === selector);
}
exports.first = first;
//# sourceMappingURL=index.js.map